@@ -1,6 +1,6 @@
 package Munin::Node::Server;
 
-# $Id: Server.pm 3164 2009-12-03 20:45:57Z janl $
+# $Id: Server.pm 3940 2010-10-06 16:54:45Z ligne $
 
 use base qw(Net::Server::Fork);
 
@@ -23,7 +23,7 @@ use Munin::Node::Utils;
 my %services;
 
 # Services that require the server to support multigraph plugins.
-my @multigraph_services;
+my (@multigraph_services, @dirtyconfig_services);
 
 # Which hosts this node's services applies to. Typically this is the
 # same as the host the node is running on, but some services query
@@ -89,15 +89,23 @@ sub _add_services_to_nodes
                 || (split /\s+/, ($host_name || ''))[1]
                 || $config->{fqdn};
 
+        # hostname checks are case insensitive, so store everything in lowercase
+        $node = lc($node);
+
         print STDERR "\tAdding to node $node\n" if $config->{DEBUG};
         push @{$nodes{$node}}, $service;
 
         # Note any plugins that require particular server capabilities.
         if (grep /^multigraph\s+/, @response) {
-            print STDERR "\tAdding to multigraph plugins\n"
-                if $config->{DEBUG};
+            print STDERR "\tAdding to multigraph plugins\n" if $config->{DEBUG};
             push @multigraph_services, $service;
         }
+        if (grep /^[A-Za-z0-9_]+\.value /, @response) {
+            # very dirty plugins -- they do a dirtyconfig even when
+            # "not allowed" by their environment.
+            print STDERR "\tAdding to dirty plugins\n" if $config->{DEBUG};
+            push @dirtyconfig_services, $service;
+        }
     }
     print STDERR "Finished configuring services\n" if $config->{DEBUG};
 
@@ -124,12 +132,21 @@ sub process_request
 
     _net_write($session, "# munin node at $config->{fqdn}\n");
 
+    my $line = '<no command received yet>';
+
     # catch and report any system errors in a clean way.
     eval {
         $timed_out = !do_with_timeout($config->{'timeout'}, sub {
-            while (defined (my $line = _net_read($session))) {
+            while (defined ($line = _net_read($session))) {
                 chomp $line;
-                _process_command_line($session, $line) or last;
+                if (! _process_command_line($session, $line)) {
+		    $line = "<finished '$line', ending input loop>";
+		    last;
+		}
+		$line = "<waiting for input from master, previous was '$line'>";
+		# Reset timeout to wait a reasonable time for input from the master
+		# Misfeature: Plugin timeout and input timeout becomes identical.
+		reset_timeout();
             }
         });
     };
@@ -139,7 +156,7 @@ sub process_request
     }
 
     if ($timed_out) {
-        logger("Connection timed out");
+        logger("Node side timeout while processing: '$line'");
     }
 
     return;
@@ -163,7 +180,7 @@ sub _process_command_line {
 
     logger ("DEBUG: Running command \"$_\".") if $config->{DEBUG};
     if (/^list\s*([0-9a-zA-Z\.\-]+)?/i) {
-        _list_services($session, $1);
+        _list_services($session, lc($1));
     }
     elsif (/^cap\s?(.*)/i) {
         _negotiate_session_capabilities($session, $1);
@@ -212,11 +229,17 @@ sub _expect_starttls {
 sub _negotiate_session_capabilities {
     my ($session, $server_capabilities) = @_;
 
-    my @node_cap = qw( multigraph );
+    my @node_cap = qw/
+        multigraph
+        dirtyconfig
+    /;
+
     $session->{server_capabilities} = {
             map { $_ => 1 } split(/ /, $server_capabilities)
     };
 
+    $ENV{MUNIN_CAP_DIRTYCONFIG} = 1 if ($session->{server_capabilities}{dirtyconfig});
+
     _net_write($session, sprintf("cap %s\n",join(" ", @node_cap)));
 }
 
@@ -233,7 +256,8 @@ sub _process_starttls_command {
     my $ca_cert    = $config->{tls_ca_certificate}
                   || "$Munin::Common::Defaults::MUNIN_CONFDIR/cacert.pem";
     my $tls_verify = $config->{tls_verify_certificate}
-                  || 'no';
+                  || 0;
+    my $tls_match  = $config->{tls_match};
 
     my $depth = $config->{tls_verify_depth};
     $depth = 5 unless defined $depth;
@@ -249,6 +273,7 @@ sub _process_starttls_command {
         tls_priv     => $key,
         tls_vdepth   => $depth,
         tls_verify   => $tls_verify,
+        tls_match    => $tls_match,
         write_fd     => fileno(STDOUT),
         write_func   => sub { print @_ },
     });
@@ -296,10 +321,13 @@ sub _list_services {
     $node ||= $config->{fqdn};
 
     if (exists $nodes{$node}) {
-        # remove any plugins that require capabilities the server doesn't provide
         my @services = @{$nodes{$node}};
+
+        # remove any plugins that require capabilities the master doesn't support
         @services = Munin::Node::Utils::set_difference(\@services, \@multigraph_services)
             unless $session->{server_capabilities}{multigraph};
+        @services = Munin::Node::Utils::set_difference(\@services, \@dirtyconfig_services)
+            unless $session->{server_capabilities}{dirtyconfig};
 
         _net_write($session, join(" ", @services));
     }
